package envoy.client.ui;

import java.awt.AWTException;
import java.awt.Image;
import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.SystemTray;
import java.awt.Toolkit;
import java.awt.TrayIcon;
import java.awt.TrayIcon.MessageType;
import java.util.HashSet;
import java.util.Set;

import envoy.client.event.Event;
import envoy.client.event.EventBus;
import envoy.client.event.EventHandler;
import envoy.client.event.MessageCreationEvent;
import envoy.exception.EnvoyException;

/**
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>StatusTrayIcon.java</strong><br>
 * Created: <strong>3 Dec 2019</strong><br>
 *
 * @author Kai S. K. Engelbart
 * @since Envoy v0.2-alpha
 */
public class StatusTrayIcon implements EventHandler {

	private TrayIcon trayIcon;

	public StatusTrayIcon() throws EnvoyException {
		if (!SystemTray.isSupported()) throw new EnvoyException("The Envoy tray icon is not supported.");

		Image img = Toolkit.getDefaultToolkit().createImage(getClass().getResource("envoy_logo.png"));
		trayIcon = new TrayIcon(img, "Envoy Client");
		trayIcon.setImageAutoSize(true);
		trayIcon.setToolTip("You are notified if you have unread messages.");

		PopupMenu popup = new PopupMenu();

		MenuItem exitMenuItem = new MenuItem("Exit");
		exitMenuItem.addActionListener((evt) -> System.exit(0));
		popup.add(exitMenuItem);

		trayIcon.setPopupMenu(popup);
		EventBus.getInstance().register(this);
	}

	/**
	 * Makes this {@link StatusTrayIcon} appear in the system tray.
	 *
	 * @throws EnvoyException
	 */
	public void show() throws EnvoyException {
		try {
			SystemTray.getSystemTray().add(trayIcon);
		} catch (AWTException e) {
			throw new EnvoyException("Could not attach Envoy tray icon to system tray.", e);
		}
	}

	/**
	 * Notifies the user of a message by displaying a pop-up.
	 */
	@Override
	public void handle(Event<?> event) {
		trayIcon.displayMessage("New message received", ((MessageCreationEvent) event).get().getContent().get(0).getText(), MessageType.INFO);
	}

	@Override
	public Set<Class<? extends Event<?>>> supports() {
		Set<Class<? extends Event<?>>> supportedEvents = new HashSet<>();
		supportedEvents.add(MessageCreationEvent.class);
		return supportedEvents;
	}
}
